@@ -1,4 +1,19 @@
 #!/bin/sh
+#
+# Copyright (c) 2012-2013 Todd C. Miller <Todd.Miller@courtesan.com>
+#
+# Permission to use, copy, modify, and distribute this software for any
+# purpose with or without fee is hereby granted, provided that the above
+# copyright notice and this permission notice appear in all copies.
+#
+# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
 
 OUTFILE="$1"
 rm -f "$OUTFILE"
@@ -20,8 +35,7 @@ case "$OUTFILE" in
 	cat >>"$OUTFILE" <<-'EOF'
 		/^\\fR0\\fR$/ {
 			N
-			s/^\\fR0\\fR\
-			minutes\.$/unlimited./
+			s/^\\fR0\\fR\nminutes\.$/unlimited./
 		}
 	EOF
 
