@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2011-2013 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -19,22 +19,31 @@
 
 #include "list.h"
 
+#define GROUP_SOURCE_ADAPTIVE	0
+#define GROUP_SOURCE_STATIC	1
+#define GROUP_SOURCE_DYNAMIC	2
+
 struct plugin_info {
     struct plugin_info *prev; /* required */
     struct plugin_info *next; /* required */
     const char *path;
     const char *symbol_name;
+    char * const * options;
+    int lineno;
 };
 TQ_DECLARE(plugin_info)
 
 /* Read main sudo.conf file. */
-void sudo_conf_read(void);
+void sudo_conf_read(const char *);
 
 /* Accessor functions. */
 const char *sudo_conf_askpass_path(void);
+const char *sudo_conf_sesh_path(void);
 const char *sudo_conf_noexec_path(void);
 const char *sudo_conf_debug_flags(void);
 struct plugin_info_list *sudo_conf_plugins(void);
 bool sudo_conf_disable_coredump(void);
+int sudo_conf_group_source(void);
+int sudo_conf_max_groups(void);
 
 #endif /* _SUDO_CONF_H */
