@@ -240,7 +240,7 @@ struct sudo_defs_types sudo_defs_table[] = {
 	def_data_verifypw,
     }, {
 	"noexec", T_FLAG,
-	N_("Preload the dummy exec functions contained in \"_PATH_SUDO_NOEXEC"),
+	N_("Preload the dummy exec functions contained in the sudo_noexec library"),
 	NULL,
     }, {
 	"ignore_local_sudoers", T_FLAG,
@@ -342,6 +342,26 @@ struct sudo_defs_types sudo_defs_table[] = {
 	"utmp_runas", T_FLAG,
 	N_("Set the user in utmp to the runas user, not the invoking user"),
 	NULL,
+    }, {
+	"privs", T_STR,
+	N_("Set of permitted privileges"),
+	NULL,
+    }, {
+	"limitprivs", T_STR,
+	N_("Set of limit privileges"),
+	NULL,
+    }, {
+	"exec_background", T_FLAG,
+	N_("Run commands on a pty in the background"),
+	NULL,
+    }, {
+	"pam_session", T_FLAG,
+	N_("Create a new PAM session for the command to run in"),
+	NULL,
+    }, {
+	"maxseq", T_UINT,
+	N_("Maximum I/O log sequence number"),
+	NULL,
     }, {
 	NULL, 0, NULL
     }
