@@ -158,6 +158,16 @@
 #define I_SET_UTMP              78
 #define def_utmp_runas          (sudo_defs_table[79].sd_un.flag)
 #define I_UTMP_RUNAS            79
+#define def_privs               (sudo_defs_table[80].sd_un.str)
+#define I_PRIVS                 80
+#define def_limitprivs          (sudo_defs_table[81].sd_un.str)
+#define I_LIMITPRIVS            81
+#define def_exec_background     (sudo_defs_table[82].sd_un.flag)
+#define I_EXEC_BACKGROUND       82
+#define def_pam_session         (sudo_defs_table[83].sd_un.flag)
+#define I_PAM_SESSION           83
+#define def_maxseq              (sudo_defs_table[84].sd_un.ival)
+#define I_MAXSEQ                84
 
 enum def_tuple {
 	never,
