@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2009-2013 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -17,7 +17,6 @@
 #include <config.h>
 
 #include <sys/types.h>
-#include <sys/param.h>
 #include <sys/uio.h>
 #ifdef HAVE_SYS_SYSMACROS_H
 # include <sys/sysmacros.h>
@@ -82,9 +81,6 @@
 #ifdef HAVE_ZLIB_H
 # include <zlib.h>
 #endif
-#ifdef HAVE_SETLOCALE
-# include <locale.h>
-#endif
 #include <signal.h>
 #ifdef HAVE_STDBOOL_H
 # include <stdbool.h>
@@ -98,6 +94,7 @@
 #include "alloc.h"
 #include "error.h"
 #include "gettext.h"
+#include "logging.h"
 #include "sudo_plugin.h"
 #include "sudo_conf.h"
 #include "sudo_debug.h"
@@ -181,8 +178,6 @@ struct search_node {
     } u;
 } *search_expr;
 
-sudo_conv_t sudo_conv;	/* NULL in non-plugin */
-
 #define STACK_NODE_SIZE	32
 static struct search_node *node_stack[32];
 static int stack_top;
@@ -204,7 +199,6 @@ extern char *get_timestr(time_t, int);
 extern int term_raw(int, int);
 extern int term_restore(int, int);
 extern void get_ttysize(int *rowp, int *colp);
-void cleanup(int);
 
 static int list_sessions(int, char **, const char *, const char *, const char *);
 static int parse_expr(struct search_node **, char **);
@@ -217,6 +211,8 @@ static int parse_timing(const char *buf, const char *decimal, int *idx, double *
 static struct log_info *parse_logfile(char *logfile);
 static void free_log_info(struct log_info *li);
 static size_t atomic_writev(int fd, struct iovec *iov, int iovcnt);
+static void sudoreplay_handler(int);
+static void sudoreplay_cleanup(void);
 
 #ifdef HAVE_REGCOMP
 # define REGEX_T	regex_t
@@ -238,12 +234,14 @@ static size_t atomic_writev(int fd, struct iovec *iov, int iovcnt);
     (s)[8] == '/' && (s)[9] == 'l' && (s)[10] == 'o' && (s)[11] == 'g' && \
     (s)[12] == '\0')
 
+__dso_public int main(int argc, char *argv[]);
+
 int
 main(int argc, char *argv[])
 {
     int ch, idx, plen, exitcode = 0, rows = 0, cols = 0;
     bool interactive = false, listonly = false, need_nlcr = false;
-    const char *id, *user = NULL, *pattern = NULL, *tty = NULL, *decimal = ".";
+    const char *decimal, *id, *user = NULL, *pattern = NULL, *tty = NULL;
     char path[PATH_MAX], buf[LINE_MAX], *cp, *ep;
     double seconds, to_wait, speed = 1.0, max_wait = 0;
     sigaction_t sa;
@@ -264,15 +262,16 @@ main(int argc, char *argv[])
     setprogname(argc > 0 ? argv[0] : "sudoreplay");
 #endif
 
-#ifdef HAVE_SETLOCALE
-    setlocale(LC_ALL, "");
+    sudoers_setlocale(SUDOERS_LOCALE_USER, NULL);
     decimal = localeconv()->decimal_point;
-#endif
     bindtextdomain("sudoers", LOCALEDIR); /* XXX - should have sudoreplay domain */
     textdomain("sudoers");
 
+    /* Register fatal/fatalx callback. */
+    fatal_callback_register(sudoreplay_cleanup);
+
     /* Read sudo.conf. */
-    sudo_conf_read();
+    sudo_conf_read(NULL);
 
     while ((ch = getopt(argc, argv, "d:f:hlm:s:V")) != -1) {
 	switch(ch) {
@@ -290,7 +289,7 @@ main(int argc, char *argv[])
 		else if (strcmp(cp, "ttyout") == 0)
 		    SET(replay_filter, 1 << IOFD_TTYOUT);
 		else
-		    errorx(1, _("invalid filter option: %s"), optarg);
+		    fatalx(_("invalid filter option: %s"), optarg);
 	    }
 	    break;
 	case 'h':
@@ -303,13 +302,13 @@ main(int argc, char *argv[])
 	    errno = 0;
 	    max_wait = strtod(optarg, &ep);
 	    if (*ep != '\0' || errno != 0)
-		errorx(1, _("invalid max wait: %s"), optarg);
+		fatalx(_("invalid max wait: %s"), optarg);
 	    break;
 	case 's':
 	    errno = 0;
 	    speed = strtod(optarg, &ep);
 	    if (*ep != '\0' || errno != 0)
-		errorx(1, _("invalid speed factor: %s"), optarg);
+		fatalx(_("invalid speed factor: %s"), optarg);
 	    break;
 	case 'V':
 	    (void) printf(_("%s version %s\n"), getprogname(), PACKAGE_VERSION);
@@ -337,13 +336,13 @@ main(int argc, char *argv[])
 	plen = snprintf(path, sizeof(path), "%s/%.2s/%.2s/%.2s/timing",
 	    session_dir, id, &id[2], &id[4]);
 	if (plen <= 0 || plen >= sizeof(path))
-	    errorx(1, _("%s/%.2s/%.2s/%.2s/timing: %s"), session_dir,
+	    fatalx(_("%s/%.2s/%.2s/%.2s/timing: %s"), session_dir,
 		id, &id[2], &id[4], strerror(ENAMETOOLONG));
     } else {
 	plen = snprintf(path, sizeof(path), "%s/%s/timing",
 	    session_dir, id);
 	if (plen <= 0 || plen >= sizeof(path))
-	    errorx(1, _("%s/%s/timing: %s"), session_dir,
+	    fatalx(_("%s/%s/timing: %s"), session_dir,
 		id, strerror(ENAMETOOLONG));
     }
     plen -= 7;
@@ -352,7 +351,7 @@ main(int argc, char *argv[])
     for (idx = 0; idx < IOFD_MAX; idx++) {
 	if (ISSET(replay_filter, 1 << idx) || idx == IOFD_TIMING) {
 	    if (open_io_fd(path, plen, io_fnames[idx], &io_fds[idx]) == -1)
-		error(1, _("unable to open %s"), path);
+		fatal(_("unable to open %s"), path);
 	}
     }
 
@@ -380,7 +379,7 @@ main(int argc, char *argv[])
     memset(&sa, 0, sizeof(sa));
     sigemptyset(&sa.sa_mask);
     sa.sa_flags = SA_RESETHAND;
-    sa.sa_handler = cleanup;
+    sa.sa_handler = sudoreplay_handler;
     (void) sigaction(SIGINT, &sa, NULL);
     (void) sigaction(SIGKILL, &sa, NULL);
     (void) sigaction(SIGTERM, &sa, NULL);
@@ -398,8 +397,7 @@ main(int argc, char *argv[])
 	if (ch != -1)
 	    (void) fcntl(STDIN_FILENO, F_SETFL, ch | O_NONBLOCK);
 	if (!term_raw(STDIN_FILENO, 1))
-	    error(1, _("unable to set tty to raw mode"));
-	iovcnt = 0;
+	    fatal(_("unable to set tty to raw mode"));
 	iovmax = 32;
 	iov = ecalloc(iovmax, sizeof(*iov));
     }
@@ -415,7 +413,7 @@ main(int argc, char *argv[])
 	char last_char = '\0';
 
 	if (!parse_timing(buf, decimal, &idx, &seconds, &nbytes))
-	    errorx(1, _("invalid timing file line: %s"), buf);
+	    fatalx(_("invalid timing file line: %s"), buf);
 
 	if (interactive)
 	    check_input(STDIN_FILENO, &speed);
@@ -498,7 +496,7 @@ main(int argc, char *argv[])
 		iovcnt = 1;
 	    }
 	    if (atomic_writev(STDOUT_FILENO, iov, iovcnt) == -1)
-		error(1, _("writing to standard output"));
+		fatal(_("writing to standard output"));
 	}
     }
     term_restore(STDIN_FILENO, 1);
@@ -527,8 +525,8 @@ delay(double secs)
       rval = nanosleep(&ts, &rts);
     } while (rval == -1 && errno == EINTR);
     if (rval == -1) {
-	error2(1, _("nanosleep: tv_sec %ld, tv_nsec %ld"),
-	    (long)ts.tv_sec, (long)ts.tv_nsec);
+	fatal_nodebug("nanosleep: tv_sec %lld, tv_nsec %ld",
+	    (long long)ts.tv_sec, (long)ts.tv_nsec);
     }
 }
 
@@ -640,7 +638,7 @@ parse_expr(struct search_node **headp, char *argv[])
 	    continue;
 	case 'c': /* command */
 	    if (av[0][1] == '\0')
-		errorx(1, _("ambiguous expression \"%s\""), *av);
+		fatalx(_("ambiguous expression \"%s\""), *av);
 	    if (strncmp(*av, "cwd", strlen(*av)) == 0)
 		type = ST_CWD;
 	    else if (strncmp(*av, "command", strlen(*av)) == 0)
@@ -665,7 +663,7 @@ parse_expr(struct search_node **headp, char *argv[])
 	    break;
 	case 't': /* tty or to date */
 	    if (av[0][1] == '\0')
-		errorx(1, _("ambiguous expression \"%s\""), *av);
+		fatalx(_("ambiguous expression \"%s\""), *av);
 	    if (strncmp(*av, "todate", strlen(*av)) == 0)
 		type = ST_TODATE;
 	    else if (strncmp(*av, "tty", strlen(*av)) == 0)
@@ -682,7 +680,7 @@ parse_expr(struct search_node **headp, char *argv[])
 	    if (av[0][1] != '\0')
 		goto bad;
 	    if (stack_top + 1 == STACK_NODE_SIZE) {
-		errorx(1, _("too many parenthesized expressions, max %d"),
+		fatalx(_("too many parenthesized expressions, max %d"),
 		    STACK_NODE_SIZE);
 	    }
 	    node_stack[stack_top++] = sn;
@@ -693,13 +691,13 @@ parse_expr(struct search_node **headp, char *argv[])
 		goto bad;
 	    /* pop */
 	    if (--stack_top < 0)
-		errorx(1, _("unmatched ')' in expression"));
+		fatalx(_("unmatched ')' in expression"));
 	    if (node_stack[stack_top])
 		sn->next = node_stack[stack_top]->next;
 	    debug_return_int(av - argv + 1);
 	bad:
 	default:
-	    errorx(1, _("unknown search term \"%s\""), *av);
+	    fatalx(_("unknown search term \"%s\""), *av);
 	    /* NOTREACHED */
 	}
 
@@ -713,17 +711,17 @@ parse_expr(struct search_node **headp, char *argv[])
 	    av += parse_expr(&newsn->u.expr, av + 1);
 	} else {
 	    if (*(++av) == NULL)
-		errorx(1, _("%s requires an argument"), av[-1]);
+		fatalx(_("%s requires an argument"), av[-1]);
 #ifdef HAVE_REGCOMP
 	    if (type == ST_PATTERN) {
 		if (regcomp(&newsn->u.cmdre, *av, REG_EXTENDED|REG_NOSUB) != 0)
-		    errorx(1, _("invalid regular expression: %s"), *av);
+		    fatalx(_("invalid regular expression: %s"), *av);
 	    } else
 #endif
 	    if (type == ST_TODATE || type == ST_FROMDATE) {
 		newsn->u.tstamp = get_date(*av);
 		if (newsn->u.tstamp == -1)
-		    errorx(1, _("could not parse date \"%s\""), *av);
+		    fatalx(_("could not parse date \"%s\""), *av);
 	    } else {
 		newsn->u.ptr = *av;
 	    }
@@ -736,11 +734,11 @@ parse_expr(struct search_node **headp, char *argv[])
 	sn = newsn;
     }
     if (stack_top)
-	errorx(1, _("unmatched '(' in expression"));
+	fatalx(_("unmatched '(' in expression"));
     if (or)
-	errorx(1, _("illegal trailing \"or\""));
+	fatalx(_("illegal trailing \"or\""));
     if (not)
-	errorx(1, _("illegal trailing \"!\""));
+	fatalx(_("illegal trailing \"!\""));
 
     debug_return_int(av - argv);
 }
@@ -783,7 +781,7 @@ match_expr(struct search_node *head, struct log_info *log)
 	    if (rc && rc != REG_NOMATCH) {
 		char buf[BUFSIZ];
 		regerror(rc, &sn->u.cmdre, buf, sizeof(buf));
-		errorx(1, "%s", buf);
+		fatalx("%s", buf);
 	    }
 	    matched = rc == REG_NOMATCH ? 0 : 1;
 #else
@@ -884,7 +882,8 @@ parse_logfile(char *logfile)
     debug_return_ptr(li);
 
 bad:
-    fclose(fp);
+    if (fp != NULL)
+	fclose(fp);
     efree(buf);
     free_log_info(li);
     debug_return_ptr(NULL);
@@ -957,7 +956,7 @@ session_compare(const void *v1, const void *v2)
     return strcmp(s1, s2);
 }
 
-/* XXX - always returns 0, calls error() on failure */
+/* XXX - always returns 0, calls fatal() on failure */
 static int
 find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 {
@@ -967,17 +966,22 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
     size_t sdlen, sessions_len = 0, sessions_size = 36*36;
     int i, len;
     char pathbuf[PATH_MAX], **sessions = NULL;
+#ifdef HAVE_STRUCT_DIRENT_D_TYPE
+    bool checked_type = true;
+#else
+    const bool checked_type = false;
+#endif
     debug_decl(find_sessions, SUDO_DEBUG_UTIL)
 
     d = opendir(dir);
     if (d == NULL)
-	error(1, _("unable to open %s"), dir);
+	fatal(_("unable to open %s"), dir);
 
     /* XXX - would be faster to chdir and use relative names */
     sdlen = strlcpy(pathbuf, dir, sizeof(pathbuf));
     if (sdlen + 1 >= sizeof(pathbuf)) {
 	errno = ENAMETOOLONG;
-	error(1, "%s/", dir);
+	fatal("%s/", dir);
     }
     pathbuf[sdlen++] = '/';
     pathbuf[sdlen] = '\0';
@@ -990,8 +994,14 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 	    (dp->d_name[1] == '.' && dp->d_name[2] == '\0')))
 	    continue;
 #ifdef HAVE_STRUCT_DIRENT_D_TYPE
-	if (dp->d_type != DT_DIR)
-	    continue;
+	if (checked_type) {
+	    if (dp->d_type != DT_DIR) {
+		/* Not all file systems support d_type. */
+		if (dp->d_type != DT_UNKNOWN)
+		    continue;
+		checked_type = false;
+	    }
+	}
 #endif
 
 	/* Add name to session list. */
@@ -1010,7 +1020,7 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 	    "%s/log", sessions[i]);
 	if (len <= 0 || len >= sizeof(pathbuf) - sdlen) {
 	    errno = ENAMETOOLONG;
-	    error(1, "%s/%s/log", dir, sessions[i]);
+	    fatal("%s/%s/log", dir, sessions[i]);
 	}
 	efree(sessions[i]);
 
@@ -1020,9 +1030,7 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 	} else {
 	    /* Strip off "/log" and recurse if a dir. */
 	    pathbuf[sdlen + len - 4] = '\0';
-#ifndef HAVE_STRUCT_DIRENT_D_TYPE
-	    if (lstat(pathbuf, &sb) == 0 && S_ISDIR(sb.st_mode))
-#endif
+	    if (checked_type || (lstat(pathbuf, &sb) == 0 && S_ISDIR(sb.st_mode)))
 		find_sessions(pathbuf, re, user, tty);
 	}
     }
@@ -1031,7 +1039,7 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
     debug_return_int(0);
 }
 
-/* XXX - always returns 0, calls error() on failure */
+/* XXX - always returns 0, calls fatal() on failure */
 static int
 list_sessions(int argc, char **argv, const char *pattern, const char *user,
     const char *tty)
@@ -1047,7 +1055,7 @@ list_sessions(int argc, char **argv, const char *pattern, const char *user,
     if (pattern) {
 	re = &rebuf;
 	if (regcomp(re, pattern, REG_EXTENDED|REG_NOSUB) != 0)
-	    errorx(1, _("invalid regex: %s"), pattern);
+	    fatalx(_("invalid regular expression: %s"), pattern);
     }
 #else
     re = (char *) pattern;
@@ -1197,12 +1205,21 @@ help(void)
 }
 
 /*
- * Cleanup hook for error()/errorx()
+ * Cleanup hook for fatal()/fatalx()
   */
-void
-cleanup(int signo)
+static void
+sudoreplay_cleanup(void)
 {
     term_restore(STDIN_FILENO, 0);
-    if (signo)
-	kill(getpid(), signo);
+}
+
+/*
+ * Signal handler for SIGINT, SIGKILL, SIGTERM, SIGHUP
+ * Must be installed with SA_RESETHAND enabled.
+ */
+static void
+sudoreplay_handler(int signo)
+{
+    term_restore(STDIN_FILENO, 0);
+    kill(getpid(), signo);
 }
